#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a quick effect

# (c) Dinasset 2023
#
#   History:
#
#   v.1.1.0: 2023-10: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
maxwh=6000

############################################################################
# Service routines
#---------------------------------------------------------------------------
# from Original drawable create the Output image
#---------------------------------------------------------------------------
def createOutimage(inImage, inLayer, widthheight):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    
   # define/calculate the scale factor for the (new) output image -------------------------
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have output size same as input size
        if realWidthHeight<=maxwh:
            widthheight = realWidthHeight
        else:
            widthheight = maxwh  # max
            gimp.message ("forced max width+height")
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size ------------------------------------------------
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
    
    # base for effect
    pdb.gimp_image_set_active_layer (newImage,  newBG)
    newBG = pdb.gimp_image_get_active_layer (newImage)
    newBG.mode = LAYER_MODE_NORMAL        
    newBG.opacity = 100.0
    
    # little mean adjustment
    (MeanV,b,c,d,e,f) = pdb.gimp_drawable_histogram (newBG,
                                 HISTOGRAM_VALUE, 0.0, 1.0)
    if MeanV<63:
        pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                         (0.0,0.0,63.0/255,191.0/255,1.0,1.0))
    elif MeanV<95:
        pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                         (0.0,0.0,95.0/255,159.0/255,1.0,1.0))
    elif MeanV>191:
        pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                         (0.0,0.0,191.0/255,63.0/255,1.0,1.0))
    elif MeanV>159:
        pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                         (0.0,0.0,159.0/255,95.0/255,1.0,1.0))
    else:
        pdb.gimp_drawable_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                         (0.0,0.0,MeanV*1.00/255,0.5,1.0,1.0))
    
    return(newImage)
#--------------------------------------------------------------------------- 
    
############################################################################
# Main
#---------------------------------------------------------------------------

def AbstractPlus (inImage, inLayer, widthheight,
                    abmode, style,
                    doFlatten):
      
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    	
    newImage = createOutimage (inImage, inLayer, widthheight)
    imgW = newImage.width
    imgH = newImage.height
    minWH = (min(imgW,imgH))
    imgsize = imgW+imgH
    
    newBGo = newImage.layers[0]
    newBGo.name = "new BG original"
    
    # create base for applying chosen effect
    # apply fx_ColorAbstractionPaint 5,10,1,0,1,0,1,0,0,0,0,0,0,0
    parms='5,10,1,0,1,0,1,0,0,0,0,0,0,0'
    pdb.plug_in_gmic_qt(newImage,newBGo, 1,2,
                        "-v - -fx_ColorAbstractionPaint "+str(parms)+" ")
    ColorAbstr = newImage.layers[0]
    ColorAbstr.name = "Color Abstract layer"
    # for better customization: let chose mode
    ABLMODE = [int(LAYER_MODE_HARDLIGHT),int(LAYER_MODE_SOFTLIGHT),int(LAYER_MODE_PIN_LIGHT),
               int(LAYER_MODE_OVERLAY),int(LAYER_MODE_HSL_COLOR),int(LAYER_MODE_LCH_COLOR),
               int(LAYER_MODE_GRAIN_MERGE),int(LAYER_MODE_DISSOLVE),int(LAYER_MODE_MERGE),
               int(LAYER_MODE_SCREEN),int(LAYER_MODE_DODGE),int(LAYER_MODE_NORMAL)]
    ABLOPAC = [float(75.00),float(99.00),float(75.00),
               float(85.00),float(50.00),float(75.00),
               float(50.00),float(75.00),float(75.00),
               float(40.00),float(55.00),float(75.00)]
    ColorAbstr.mode = int(ABLMODE[abmode])
    ColorAbstr.opacity = float(ABLOPAC[abmode])
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # for better testing: do not merge
    baseforeffect = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                     "Base for effect")
    newImage.add_layer (baseforeffect, 0)
    #baseforeffect = pdb.gimp_image_merge_down (newImage, ColorAbstr,
    #                                            CLIP_TO_IMAGE)
    baseforeffect.name = "Base for effect"
    baseforeffect.mode = LAYER_MODE_NORMAL
    baseforeffect.opacity = 100.0
    #pdb.gimp_drawable_equalize (baseforeffect, False)
    
    # apply chosen effect
    # A=thru G'MIC filter: gui_rep_frblur
    if style==0:  #effect A
        # gui_rep_frblur 9,18,7,180,0,1,0,0,50,50 (#menu degradation/fragment_blur)
        parms='9,18,7,180,0,1,0,0,50,50'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
                            "-v - -gui_rep_frblur "+str(parms)+" ")     
        
        ActionLayer.name = "Effect A2 action: gui_rep_frblur"
        ActionLayer.mode = LAYER_MODE_LUMINANCE
        ActionLayer.opacity = 65.0
        #pdb.gimp_drawable_equalize (ActionLayer, False)
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect A2 (Fragment Blur)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # B=thru G'MIC filter: fx_rep_blur_splinter
    elif style==1:  #effect B
        # fx_rep_blur_splinter 30,3,-100,0,0,0,3,1,0,50,50
        parms='30,3,-100,0,0,0,3,1,0,50,50'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_rep_blur_splinter "+str(parms)+" ")     
        
        ActionLayer.name = "Effect B2 action: fx_rep_blur_splinter"
        ActionLayer.mode = LAYER_MODE_LUMINANCE
        ActionLayer.opacity = 55.0
        pdb.gimp_drawable_equalize (ActionLayer, False)
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect B2 (Blur Splinter")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # C=thru G'MIC filter: fx_blur_angular
    elif style==2:  #effect C
        # fx_blur_angular 1,50,50,0,1,7,0
        parms='1,50,50,0,1,7,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_blur_angular "+str(parms)+" ")     
        
        ActionLayer.name = "Effect C2 action: fx_blur_angular"
        ActionLayer.mode = LAYER_MODE_GRAIN_MERGE
        ActionLayer.opacity = 35.0
        pdb.gimp_drawable_equalize (ActionLayer, False)
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect C2 (Blur Angular)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # D=thru G'MIC filter: fx_gaussian_blur
    elif style==3:  #effect D
        # fx_gaussian_blur 20,0,0,1,0,0,0,50,50
        parms='20,0,0,1,0,0,0,50,50'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_gaussian_blur "+str(parms)+" ")     
        
        ActionLayer.name = "Effect D2 action: fx_gaussian_blur"
        ActionLayer.mode = LAYER_MODE_SCREEN
        ActionLayer.opacity = 95.0
        pdb.gimp_drawable_equalize (ActionLayer, False)
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect D2 (Blur Gaussian)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # E=thru G'MIC filter: fx_mess_with_bits  
    elif style==4:  #effect E
        # fx_mess_with_bits 1,25,1,1,0,15,100,0,0,50,50
        parms='1,25,1,1,0,15,100,0,0,50,50'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_mess_with_bits "+str(parms)+" ")     
        
        ActionLayer.name = "Effect E2 action: fx_mess_with_bits"
        ActionLayer.mode = LAYER_MODE_OVERLAY
        ActionLayer.opacity = 45.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect E2 (Mess with Bits)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # F=thru G'MIC filter: fx_blur_radial  
    elif style==5:  #effect F
        # fx_blur_radial 20,0,0,500,0,7,0
        parms='20,0,50,500,0,7,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_blur_radial "+str(parms)+" ")     
        
        ActionLayer.name = "Effect F2 action: fx_blur_radial"
        ActionLayer.mode = LAYER_MODE_OVERLAY
        ActionLayer.opacity = 75.0
        pdb.gimp_drawable_equalize (ActionLayer, False)
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect F2 (Blur Radial)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # G=thru G'MIC filter: fx_twirl  
    elif style==6:  #effect G
        # fx_twirl 5,0,0,2
        parms='5,0,0,2'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_twirl  "+str(parms)+" ")     
        
        ActionLayer.name = "Effect G action: fx_twirl "
        # add secondary effect
        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -gtutor_hairlock 0.5,0.5,0.5,255,255,0,255,0.5,45,0.5,0.5,0 ")
        ActionLayer.mode = LAYER_MODE_LINEAR_LIGHT
        ActionLayer.opacity = 50.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect G (Twirl)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # H=thru G'MIC filter: fx_textured_glass  
    elif style==7:  #effect H
        # fx_textured_glass 40,40,1,1,0,2,0,0,50,50
        parms='40,40,1,1,0,2,0,0,50,50'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_textured_glass  "+str(parms)+" ")     
        
        ActionLayer.name = "Effect H action: fx_textured_glass "
        ActionLayer.mode = LAYER_MODE_HARDLIGHT
        ActionLayer.opacity = 35.0
        pdb.gimp_drawable_equalize (ActionLayer, False)
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect H (Textured Glass)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # I=thru G'MIC filter: fx_triangular_pattern  
    elif style==8:  #effect I
        # fx_triangular_pattern 43,7,4,4,4,0,0,0,100,0,0,0,160,1
        parms='43,7,4,4,4,0,0,0,100,0,0,0,160,1'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_triangular_pattern  "+str(parms)+" ")     
        
        ActionLayer.name = "Effect I action: fx_triangular_pattern "
        ActionLayer.mode = LAYER_MODE_GRAIN_MERGE
        ActionLayer.opacity = 35.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect I (Triangular Pattern)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # J=thru G'MIC filter: fx_circle_art  
    elif style==9:  #effect J
        # fx_circle_art 1,15,0.5,8,1,1,15,0,0,0.5,1,1,1,1
        parms='1,15,0.5,8,1,1,15,0,0,0.5,1,1,1,1'
        pdb.gimp_image_scale (newImage, minWH, minWH)   # render a square for circle_art
        pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,2,  #let create the new layer
        "-v - -fx_circle_art  "+str(parms)+" ")
        ActionLayer = newImage.layers[0]
        pdb.gimp_image_scale (newImage, imgW, imgH) # back
        ActionLayer.name = "Effect J action: fx_circle_art "
        ActionLayer.mode = LAYER_MODE_MULTIPLY
        ActionLayer.opacity = 90.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect J (Circle Art)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # K=thru G'MIC filter: disco  
    elif style==10:  #effect K
        # disco 10,15,20,250
        parms='10,15,20,250'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -disco  "+str(parms)+" ")
        
        ActionLayer.name = "Effect K action: disco "
        ActionLayer.mode = LAYER_MODE_GRAIN_EXTRACT
        ActionLayer.opacity = 15.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect K (Disco)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_MULTIPLY
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # L=thru G'MIC filter: fx_memoakten_algorithm_a  
    elif style==11:  #effect L
        # fx_memoakten_algorithm_a 0,20,30,30,2,50,10,50,40,3,100,1,
        # 0,0,0,255,255,255,255,0,0,255,128,0,255,255,0,0,0,0
        parms='0,20,30,30,2,50,10,50,40,3,100,1,'
        parms=parms+'0,0,0,255,255,255,255,0,0,255,128,0,255,255,0,0,0,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_memoakten_algorithm_a  "+str(parms)+" ")
        
        ActionLayer.name = "Effect L action: fx_memoakten_algorithm_a "
        ActionLayer.mode = LAYER_MODE_SOFTLIGHT
        ActionLayer.opacity = 60.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect L (Memoakten Algorithm A)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # M=thru G'MIC filter: fx_corner_gradient  
    elif style==12:  #effect M
        # fx_corner_gradient 255,255,255,128,255,0,0,255,0,255,0,255,0,0,255,255,1
        parms='255,255,255,128,255,0,0,255,0,255,0,255,0,0,255,255,1'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_corner_gradient "+str(parms)+" ")
        
        ActionLayer.name = "Effect M action: fx_corner_gradient "
        ActionLayer.mode = LAYER_MODE_LUMINANCE
        ActionLayer.opacity = 35.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect M (Corner Gradient)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # N=thru G'MIC filter: fx_random_gradient  
    elif style==13:  #effect N
        # fx_random_gradient 96,0,0,128,128,128,1
        parms='96,0,0,128,128,128,1'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_random_gradient "+str(parms)+" ")
        
        ActionLayer.name = "Effect M action: fx_random_gradient "
        ActionLayer.mode = LAYER_MODE_HARDLIGHT
        ActionLayer.opacity = 65.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect N (Random Gradient)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_GRAIN_MERGE
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # O=thru G'MIC filter: fx_hypotrochoid  
    elif style==14:  #effect O
        # fx_hypotrochoid 37,100,74,80,0.5,255,255,100,255,1
        parms='37,100,74,100,0.5,255,255,100,255,1'
        pdb.gimp_image_scale (newImage, minWH, minWH)   # render a square for hypotrochoid
        pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,2,  #let create the new layer
        "-v - -fx_hypotrochoid  "+str(parms)+" ")
        ActionLayer = newImage.layers[0]
        pdb.gimp_image_scale (newImage, imgW, imgH) # back
        ActionLayer.name = "Effect O action: fx_hypotrochoid "
        ActionLayer.mode = LAYER_MODE_HARDLIGHT
        ActionLayer.opacity = 45.0
        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,   # over effect
        "-v - -fx_gradient2rgb 0,0,100,0,0,0,50,50 ")
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect O (Hypotrochoid)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_ADDITION
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # P=thru G'MIC filter: KittyRings  
    elif style==15:  #effect P
        # KittyRings 30,8,0,1,0,0,255,255,170,0
        parms='30,8,0,1,0,0,255,255,170,0'
        pdb.gimp_image_scale (newImage, minWH, minWH)   # render a square for hypotrochoid
        pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,2,  #let create the new layer
        "-v - -KittyRings "+str(parms)+" ")
        ActionLayer = newImage.layers[0]
        pdb.gimp_image_scale (newImage, imgW, imgH) # back
        ActionLayer.name = "Effect P action: KittyRings "
        ActionLayer.mode = LAYER_MODE_HSV_SATURATION
        ActionLayer.opacity = 25.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect P (KittyRings)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_GRAIN_MERGE
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # Q=thru G'MIC filter: fx_lissajous  
    elif style==16:  #effect Q
        # fx_lissajous 4096,0.98,0.98,3,8,7,0,0,0,0,0,0,0,0,255,170,0,255
        parms='4096,0.98,0.98,3,8,7,0,0,0,0,0,0,0,0,255,170,0,255'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_lissajous "+str(parms)+" ")
        
        ActionLayer.name = "Effect Q action: fx_lissajous "
        ActionLayer.mode = LAYER_MODE_HSV_VALUE
        ActionLayer.opacity = 50.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect Q (Lissajous)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # R=thru G'MIC filter: fx_rep_nebulous  
    elif style==17:  #effect R
        # fx_rep_nebulous 0,4,10,10,0,0,0,1,0,0,1,0,0,0,0,2,0,1,3
        parms='0,4,10,10,0,0,0,1,0,0,1,0,0,0,0,2,0,1,3'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_rep_nebulous  "+str(parms)+" ")
        
        ActionLayer.name = "Effect R action: fx_rep_nebulous  "
        ActionLayer.mode = LAYER_MODE_LCH_CHROMA    # was _COLOR
        ActionLayer.opacity = 55.0                  # was 75.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect R (Nebulous)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # S=thru G'MIC filter: fx_neon_lightning  
    elif style==18:  #effect S
        # fx_neon_lightning 50,50,0,50,50,100,100,1,3,130,80,50,1,0
        parms='50,50,0,50,50,100,100,1,3,130,80,50,1,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_neon_lightning  "+str(parms)+" ")
        
        ActionLayer.name = "Effect S action: fx_neon_lightning  "
        ActionLayer.mode = LAYER_MODE_SCREEN
        ActionLayer.opacity = 35.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect S (NeonLightning)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # T=thru G'MIC filter: fx_plasma
    elif style==19:  #effect T
        # fx_plasma 0.5,0,8,0,0,128,128,128
        parms='0.5,0,8,0,0,128,128,128'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_plasma  "+str(parms)+" ")
        
        ActionLayer.name = "Effect T action: fx_plasma "
        ActionLayer.mode = LAYER_MODE_HSL_COLOR
        ActionLayer.opacity = 95.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect T (Plasma)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # U=thru G'MIC filter: fx_rep_mj_prn
    elif style==20:  #effect U
        # fx_rep_mj_prn 8,2,1,1,0,1,50,50,1,0,0,0
        parms='8,2,1,1,0,1,50,50,1,0,0,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_rep_mj_prn "+str(parms)+" ")
        
        ActionLayer.name = "Effect U action: fx_rep_mj_prn "
        ActionLayer.mode = LAYER_MODE_GRAIN_EXTRACT
        ActionLayer.opacity = 10.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect U (PrNoise)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # V=thru G'MIC filter: fx_rep_mj_prn
    elif style==21:  #effect V
        # fx_shadebobs 50,5,200,1,-1,2,1,0.8,0,3
        parms='50,5,200,1,-1,2,1,0.8,0,3'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_shadebobs "+str(parms)+" ")
        
        ActionLayer.name = "Effect V action: fx_shadebobs "
        ActionLayer.mode = LAYER_MODE_LINEAR_BURN
        ActionLayer.opacity = 50.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect V (Shadebobs)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # W=thru G'MIC filter: spiral_RGB
    elif style==22:  #effect W
        # spiral_RGB 1,3,5,0
        parms='1,3,5,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -spiral_RGB "+str(parms)+" ")
        
        ActionLayer.name = "Effect W action: spiral_RGB "
        ActionLayer.mode = LAYER_MODE_OVERLAY
        ActionLayer.opacity = 50.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect W (SpiralRGB)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # X=thru G'MIC filter: fx_turbulence
    elif style==23:  #effect X
        # fx_turbulence 128,6,4,0,0
        parms='128,6,4,0,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -fx_turbulence "+str(parms)+" ")
        
        ActionLayer.name = "Effect X action: fx_turbulence "
        ActionLayer.mode = LAYER_MODE_HARDLIGHT
        ActionLayer.opacity = 65.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect X (Turbulence)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # X=thru G'MIC filter: Twisted_Rays
    elif style==24:  #effect Y
        # Twisted_Rays 800,                                         # square dimension 
        #     1,0.2,2,5,21,0,0,
        #     0,0,255,255,0,255,255,255,0,255,0,255,255,255,0,255,  # 4 colours+
        #     255,0,0,255,255,0,255,255,255,255,255,255,            # 3 colours
        #     0,0,0,0,255,0,2,2,200,0,5,0,0
        parms='800,'
        parms=parms+'1,0.2,2,5,21,0,0,'
        parms=parms+'0,0,255,255,0,255,255,255,0,255,0,255,255,255,0,255,'
        parms=parms+'255,0,0,255,255,0,255,255,255,255,255,255,'
        parms=parms+'0,0,0,0,255,0,2,2,200,0,5,0,0'
        pdb.gimp_image_scale (newImage,800, 800)   # render a square for Twisted_Rays
        pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,2,  #let create the new layer
        "-v - -Twisted_Rays "+str(parms)+" ")
        ActionLayer = newImage.layers[0]
        pdb.gimp_image_scale (newImage, imgW, imgH) # back
        
        ActionLayer.name = "Effect Y action: Twisted_Rays "
        ActionLayer.mode = LAYER_MODE_LCH_COLOR
        ActionLayer.opacity = 85.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect Y (Twisted_Rays)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 50.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    # apply chosen effect
    # X=thru G'MIC filter: gcd_wiremap
    elif style==25:  #effect Z
        # gcd_wiremap 100,100,0.5,0.5,0
        parms='100,100,0.5,0.5,0'
        ActionLayer = baseforeffect.copy()
        newImage.add_layer (ActionLayer, 0)

        pdb.plug_in_gmic_qt(newImage,ActionLayer, 1,0,
        "-v - -gcd_wiremap "+str(parms)+" ")
        
        ActionLayer.name = "Effect Z action: gcd_wiremap "
        ActionLayer.mode = LAYER_MODE_LUMINANCE
        ActionLayer.opacity = 35.0
        
        EffectLayer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                       "Effect Z (Wiremap)")
        newImage.add_layer (EffectLayer, 0)
        EffectLayer.mode = LAYER_MODE_NORMAL
        EffectLayer.opacity = 100.0
          
    #pdb.gimp_display_new (newImage)
    #return
    
    else:    #tbd
        pdb.gimp_message ("Effect ABSTRACT ONLY") 
           
        pdb.gimp_display_new (newImage)
        return
                                                                        
    # COMMON ******************************************************
    
    # outcome
    
    outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (outcome, 0)
    outcome.name = st+" "+oname+" with a Quick "+EffectLayer.name
    ABLMODET = ["LAYER_MODE_HARDLIGHT","LAYER_MODE_SOFTLIGHT","LAYER_MODE_PIN_LIGHT",
                "LAYER_MODE_OVERLAY","LAYER_MODE_HSL_COLOR","LAYER_MODE_LCH_COLOR",
                "LAYER_MODE_GRAIN_MERGE","LAYER_MODE_DISSOLVE","LAYER_MODE_MERGE",
                "LAYER_MODE_SCREEN","LAYER_MODE_DODGE","LAYER_MODE_NORMAL"]
    outcome.name = outcome.name+" (abstr.mode="+str(ABLMODET[abmode])
        
    #pdb.gimp_display_new (newImage)
    #return
        
 
    if doFlatten==True:
        pdb.gimp_image_remove_layer (newImage, newBGo)
        pdb.gimp_image_remove_layer (newImage, ColorAbstr)
        pdb.gimp_image_remove_layer (newImage, baseforeffect)
        pdb.gimp_image_remove_layer (newImage, ActionLayer)
        pdb.gimp_image_remove_layer (newImage, EffectLayer)        
          
    pdb.gimp_display_new (newImage)
    return
             
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Quick Effect"

register(
    "AbstractPlus",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Abstract Plus...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
         "\n (0=same as input)", 3000, (0, 6000, 100)),
        (PF_OPTION, "abmode", "Abstract layer mode", 0, ("HARDLIGHT",
                                                         "SOFTLIGHT",
                                                         "PIN_LIGHT",
                                                         "OVERLAY",
                                                         "HSL_COLOR",
                                                         "LCH_COLOR",
                                                         "GRAIN_MERGE",
                                                         "DISSOLVE",
                                                         "MERGE",
                                                         "SCREEN",
                                                         "DODGE",
                                                         "NORMAL",
                                                        )),
        (PF_OPTION, "quickeffect", "Choose quick effect: Abstract plus ", 0,
                     ["A (Fragment Blur)","B (Blur Splinter","C (Blur Angular)",
                      "D (Blur Gaussian)","E (Mess with Bits)","F (Blur Radial)",
                      "G (Twirl)","H (Textured Glass)","I (Triangular Pattern)",
                      "J (Circle Art)","K (Disco)","L (Memoakten Algorithm A)",
                      "M (Corner Gradient)","N (Random Gradient)","O (Hypotrochoid)",
                      "P (KittyRings)","Q (Lissajous)","R (Nebulous)",
                      "S (Neon Lightning)","T (Plasma)","U (PrNoise)",
                      "V (Shadebobs)","W (Spiral RGB)","X (Turbulence)",
                      "Y (Twisted Rays)","Z (Wiremap)",
                      "* (test only abstract part)",
                      ]),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    AbstractPlus,
    menu=menu
)

main()
